package com.college

import com.college.Requests.AuthorizeUserRequest
import com.college.Requests.RegisterUserRequest
import io.ktor.http.*
import io.ktor.resources.*
import io.ktor.serialization.kotlinx.json.*
import io.ktor.server.application.*
import io.ktor.server.plugins.contentnegotiation.*
import io.ktor.server.plugins.swagger.*
import io.ktor.server.request.*
import io.ktor.server.resources.*
import io.ktor.server.response.*
import io.ktor.server.routing.*
import kotlinx.serialization.Serializable
import org.jetbrains.exposed.sql.*

fun Application.configureRouting() {

    val userService = UserService()
    routing {
        route("user") {
            get {
                val users = userService.getAllUsers()
                call.respond(users)
            }
            put("{user_id}/clicks") {
                val id = call.parameters["user_id"]?.toInt()
                if (id != null) {
                    userService.registerClickByUser(id)
                    call.respond(HttpStatusCode.OK)
                }
                call.respond(HttpStatusCode.NotFound)
            }


        }
        route("authorize") {
            post("register") {
                val user = call.receive<RegisterUserRequest>()
                val newUser: User? = userService.createUser(user)
                if (newUser == null) call.respond(HttpStatusCode.Conflict)
                call.respond(HttpStatusCode.OK, newUser!!)
            }
            post("auth") {
                val user = call.receive<AuthorizeUserRequest>()
                val newUser: User? = userService.authorizeUser(user)
                if (newUser == null) call.respond(HttpStatusCode.NotFound)
                call.respond(HttpStatusCode.OK, newUser!!)
            }
        }
    }
}
