package com.college

import com.college.Requests.AuthorizeUserRequest
import com.college.Requests.RegisterUserRequest
import kotlinx.coroutines.Dispatchers
import kotlinx.serialization.Serializable
import org.jetbrains.exposed.dao.id.IntIdTable
import org.jetbrains.exposed.sql.*
import org.jetbrains.exposed.sql.transactions.experimental.newSuspendedTransaction
import org.jetbrains.exposed.sql.transactions.transaction

@Serializable
data class User(val id:Int, val nickname: String, val password: String, val clicks:Int, var url:String? = null )

class UserService() {
    object Users : IntIdTable() {
        val nickname = text("name")
        val clicks = integer("clicks")
        val password = text("password")
        val url = text("url").nullable()
    }


    suspend fun getAllUsers(): List<User> = dbQuery {
        Users.selectAll().map { it -> User(
            id = it[Users.id].value,
            nickname = it[Users.nickname],
            password = it[Users.password],
            clicks = it[Users.clicks],
            url = it[Users.url]
        ) }
    }

    suspend fun authorizeUser(userRequest:AuthorizeUserRequest): User? = dbQuery{
        val user = Users.selectAll().where{
            Users.nickname eq userRequest.login
        }.andWhere {
            Users.password eq userRequest.password
        }.map { it -> User(
            id = it[Users.id].value,
            nickname = it[Users.nickname],
            password = it[Users.password],
            clicks = it[Users.clicks],
            url = it[Users.url]
        )
        };
        return@dbQuery user.firstOrNull()
    }

    suspend fun createUser(user: RegisterUserRequest): User? = dbQuery {
        val user = Users.insertReturning {
            it[nickname] = user.login
            it[password] = user.password
            it[url] = user.url
            it[clicks] = 0
        }.map {
            it -> User(
            id = it[Users.id].value,
            nickname = it[Users.nickname],
            password = it[Users.password],
            clicks = it[Users.clicks],
            url = it[Users.url],)
        }
        return@dbQuery user.firstOrNull()
    }

    suspend fun registerClickByUser(userID: Int) {
        dbQuery {
            Users.update({ Users.id eq userID }) {
                with(SqlExpressionBuilder){
                    it[clicks] = clicks + 1
                }
            }
        }
    }

    private suspend fun <T> dbQuery(block: suspend () -> T): T =
        newSuspendedTransaction(Dispatchers.IO) { block() }
}

